@@ -1,6 +1,6 @@
-class SymbolNotFoundException(Exception):
-    """Symbol not found from loaded modules."""
-
-
 class EmulatorCrashedException(Exception):
     """Emulate failed when calling functions."""
+
+
+class SymbolMissingException(Exception):
+    """Symbol not found from loaded modules."""
