@@ -4,7 +4,7 @@ from typing import List, Optional
 
 @dataclass
 class Symbol:
-    """The symbol struct."""
+    """The symbol object."""
 
     address: int
     name: str
@@ -12,7 +12,7 @@ class Symbol:
 
 @dataclass
 class Module:
-    """The module struct."""
+    """The module object."""
 
     base: int
     size: int
@@ -22,30 +22,22 @@ class Module:
 
 @dataclass
 class Location:
-    """The location struct."""
+    """The location object."""
 
     address: int
     module: Optional[Module]
 
     def __repr__(self):
         if not self.module:
-            return f"0x{self.address}"
-        return f"0x{(self.address - self.module.base):x}@{self.module.name}"
+            return f"0x{self.address:x}"
 
-
-@dataclass
-class BackTrace:
-    """The back trace struct."""
-
-    locations: List[Location]
-
-    def __repr__(self):
-        return str(self.locations)
+        offset = self.address - self.module.base
+        return f"0x{offset:x}@{self.module.name}"
 
 
 @dataclass
 class Pool:
-    """The memory pool struct.`"""
+    """The pool object."""
 
     address: int
     size: int
@@ -58,3 +50,19 @@ class Pool:
     @property
     def block_size(self) -> int:
         return self.size // self.block_num
+
+
+@dataclass
+class Arch:
+    """The arch object."""
+
+    name: str
+    reg_size: int
+
+    reg_sp: int
+    reg_fp: int
+    reg_lr: int
+    reg_pc: int
+
+    reg_args: List[int]
+    reg_ret: int
